-- $Id: scopechan.vhd,v 1.6 2011/10/18 16:05:10 jrothwei Exp $
-- Copyright 2011 Joseph Rothweiler, Sensicomm LLC. Started 28Aug2011.
-- Implement one channel of the oscilloscope function, including
--   A/D converter capture
--   Optional desampling using CIC filter
--   Capture in dual port RAM buffer.

library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;     -- Makes the "div+1" instruction work.
use ieee.std_logic_unsigned.all;

entity scopechan is
  Port (
    CLK_48M   : in  STD_LOGIC; --  48 MHz USB clock.
    CLK_100M  : in  STD_LOGIC; -- 100 MHz system clock.
    downcmd   : in  STD_LOGIC; -- CIC strobe, 1 CLK_100M cycle.
    rt_shift  : in  STD_LOGIC_VECTOR( 5 downto 0); -- Right shift scaling after CIC.
    cic_out_pos  : out STD_LOGIC_VECTOR(39 downto 0);
    cic_out_neg  : out STD_LOGIC_VECTOR(39 downto 0);
    a2d_in_max   : out STD_LOGIC_VECTOR( 7 downto 0);
    a2d_in_min   : out STD_LOGIC_VECTOR( 7 downto 0);
    triggerlevel : in  STD_LOGIC_VECTOR( 7 downto 0);
    triggerline  : out STD_LOGIC;
    write_addr1  : in  STD_LOGIC_VECTOR(10 downto 0); -- Increments on downcmd and cic_write.

    a2d_data  : in  STD_LOGIC_VECTOR( 7 downto 0); -- A/D input data, at CLK_100M rate.
    cic_write : in  STD_LOGIC;                     -- Enables CIC output write to RAM.
    hilo_reset: in  STD_LOGIC;                     -- Reset for the high-low bit finder.
    read_addr : in  STD_LOGIC_VECTOR(10 downto 0); -- Address for read channel.
    read_data : out STD_LOGIC_VECTOR( 7 downto 0)  -- Data for read channel.
  );
end scopechan;
-------------------------------------------------------------------------------
architecture rtl of scopechan is
  component right64 is
    Port (
      clk      : in STD_LOGIC;
      leftpos  : in  STD_LOGIC_VECTOR( 5 downto 0); -- LSB position.
      data_in  : in  STD_LOGIC_VECTOR(63 downto 0); -- Input word.
      data_out : out STD_LOGIC_VECTOR(63 downto 0)  -- Output byte. Zero-padded on left.
    );
  end component;
  component cic3_downchan is
    generic (
      inwidth    : integer ; -- Length of 2's complement input, including sign bit.
      o3width    : integer   -- Length of output, typically inwidth+3*log2(downfactor).
    ) ;
    port (
      clk      : in  STD_LOGIC; -- System clock at the input sampling rate.
      downcmd  : in  STD_LOGIC; -- High for 1 clk cycle when a new outsig is generated.
      insig    : in  STD_LOGIC_VECTOR(inwidth-1 downto 0); -- Sampled every clock.
      outsig   : out STD_LOGIC_VECTOR(o3width-1 downto 0)  -- Changes after downcmd cycle.
    );
  end component;
  component xilinx_2kx9 is
    Port (
      clock1   : in  STD_LOGIC;
      write1   : in  STD_LOGIC;
      addr1    : in  STD_LOGIC_VECTOR (10 downto 0);
      data1in  : in  STD_LOGIC_VECTOR ( 8 downto 0);
      data1out : out STD_LOGIC_VECTOR ( 8 downto 0);
      --
      clock2   : in  STD_LOGIC;
      write2   : in  STD_LOGIC;
      addr2    : in  STD_LOGIC_VECTOR (10 downto 0);
      data2in  : in  STD_LOGIC_VECTOR ( 8 downto 0);
      data2out : out STD_LOGIC_VECTOR ( 8 downto 0)
    );
  end component;
  signal mem_clock1   : STD_LOGIC;
  signal mem_write1   : STD_LOGIC;
  signal mem_data1in  : STD_LOGIC_VECTOR( 8 downto 0);
  signal mem_data1out : STD_LOGIC_VECTOR( 8 downto 0);
  signal mem_clock2   : STD_LOGIC;
  signal mem_write2   : STD_LOGIC := '0';
  signal mem_addr2    : STD_LOGIC_VECTOR(10 downto 0);
  signal mem_data2in  : STD_LOGIC_VECTOR( 8 downto 0);
  signal mem_data2out : STD_LOGIC_VECTOR( 8 downto 0);
  signal cic_downcmd  : STD_LOGIC;
  signal cic_in       : STD_LOGIC_VECTOR( 7 downto 0);
  signal cic_out      : STD_LOGIC_VECTOR(39 downto 0);
  signal cic_out_pos_local : STD_LOGIC_VECTOR(39 downto 0);
  signal cic_out_neg_local : STD_LOGIC_VECTOR(39 downto 0);
  signal rt_data_in   : STD_LOGIC_VECTOR(63 downto 0);
  signal rt_data_out  : STD_LOGIC_VECTOR(63 downto 0);
  signal a2d_in_max_local : STD_LOGIC_VECTOR( 7 downto 0);
  signal a2d_in_min_local : STD_LOGIC_VECTOR( 7 downto 0);
begin -------------------------------------------------------------------------
  ----------------------------------------
  -- Modules.
  -- --------------------------------------
  cic: cic3_downchan generic map (
    inwidth    =>  8,
    o3width    => 40  -- FIXME: want 48.
  ) port map (
    clk      => CLK_100M,
    downcmd  => cic_downcmd,
    insig    => cic_in,
    outsig   => cic_out
  );
  rt64: right64 port map (
      clk      => CLK_100M,
      leftpos  => rt_shift,
      data_in  => rt_data_in,
      data_out => rt_data_out
    );

  mem2kx9: xilinx_2kx9 port map (
    clock1   => mem_clock1,
    write1   => mem_write1,
    addr1    => write_addr1,
    data1in  => mem_data1in,
    data1out => mem_data1out,
    --
    clock2   => mem_clock2,
    write2   => mem_write2,
    addr2    => mem_addr2,
    data2in  => mem_data2in,
    data2out => mem_data2out
  );
  ----------------------------------------
  -- Connect up the modules.
  ----------------------------------------
  mem_clock2 <= CLK_48M;
  mem_clock1 <= CLK_100M;
  mem_addr2 <=  read_addr;
  read_data <= mem_data2out(7 downto 0);
  ----------------------------------------
  process(CLK_100M) begin
    if rising_edge(CLK_100M) then
      cic_in <=  not a2d_data(7) & a2d_data(6 downto 0);
      cic_downcmd <= downcmd;
      if hilo_reset='1' then
        a2d_in_max_local <= X"80";
        a2d_in_min_local <= X"7f";
      else
        if signed(a2d_in_max_local)<signed(cic_in) then
          a2d_in_max_local<=cic_in;
        end if;
        if signed(a2d_in_min_local)>signed(cic_in) then
          a2d_in_min_local<=cic_in;
        end if;
      end if;
    end if;
    a2d_in_max<= a2d_in_max_local;
    a2d_in_min<= a2d_in_min_local;
  end process;
  ----------------------------------------
  -- Right shift the cic output.
  -- FIXME: Need to add overflow checking here.
  -- Also, This is working at 100 MHZ; really need
  -- only work at the downconverted (downcmd) rate.
  ----------------------------------------
  process(CLK_100M) begin
    if rising_edge(CLK_100M) then
      rt_data_in(cic_out'length-1 downto 0) <= cic_out;
      rt_data_in(rt_data_in'length-1 downto cic_out'length) <=
        (others => '0') ; -- FIXME: Sign extend??
    end if;
  end process;
  ----------------------------------------
  -- Save data into the RAM at the CIC output rate.
  -- If cic_downcmd is high for every clock cycle then the output rate is 100MHz.
  process(CLK_100M) begin
    if rising_edge(CLK_100M) then
      if(cic_downcmd='1') then
        -- mem_data1in <= write_addr1(8 downto 0);
        -- mem_data1in <= cic_out(8 downto 0); -- FIXME: select bits.
        mem_data1in <= rt_data_out(8 downto 0);
        mem_write1 <= cic_write; -- Write if enabled.
        if rt_data_out(7 downto 0)>triggerlevel then
          triggerline <= '1';
        else
          triggerline <= '0';
        end if;
      else
        mem_write1 <= '0';
      end if;
      -- Increment the address when we do a write.
      -- This happens externally now.
      -- if(mem_write1='1') then
      --   write_addr1 <= write_addr1 + 1;
      -- end if;
      -- Range tracking, to check for overflow. Options here are to use
      -- a priority encoder to find the MSB or a max and min circuit.
      -- Instead, I just and and or the bits to keep track of where the MSB is.
      if(hilo_reset='1') then
        cic_out_pos_local <= (others => '0');
        cic_out_neg_local <= (others => '1');
      elsif cic_downcmd='1' then
        if cic_out(cic_out'length-1)='0' then
          cic_out_pos_local <= cic_out_pos_local or cic_out;
        else
          cic_out_neg_local <= cic_out_neg_local and cic_out;
        end if;
      end if;
    end if;
    cic_out_pos <= cic_out_pos_local;
    cic_out_neg <= cic_out_neg_local;
  end process;
end rtl;
